<?php


function frontrom_clean( $var ) {
    if ( is_array( $var ) ) {
        return array_map( 'frontrom_clean', $var );
    } else {
        return is_scalar( $var ) ? sanitize_text_field( wp_unslash( $var ) ) : $var;
    }
}

/**
 * The main logging function
 *
 * @since 0.1
 * @uses error_log
 * @param string $type type of the error. e.g: debug, error, info
 * @param string $msg
 */
function frontrom_log( $type = '', $msg = '' ) {
    $ouput_path = FRONTROM_PATH;

    if ( WP_DEBUG == true ) {
        $msg = sprintf( "[%s][%s] %s\n", date( 'd.m.Y h:i:s' ), $type, print_r($msg, true) );
        error_log( $msg, 3, $ouput_path . '/tmp/debug.log' );
    }
}

function frontrom_generate_random_string( $length = 10 ) {
    return substr(
            str_shuffle(
                str_repeat(
                    $x = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ', 
                    ceil( $length/strlen($x) ) 
                )
            ), 
            1,
            $length
        );
}

function frontrom_user_can_access_front_end_editor() {
    
    if ( ! current_user_can( 'edit_posts' ) ) {
        return false;
    }

    if ( 
        ! isset( $_GET['frontrom'] ) 
            &&
        ! isset( $_GET['secret'] )
            &&
        ! isset( $_GET['_wpnonce'] )
    ) {
        return false;
    }

    if ( sanitize_text_field( $_GET['frontrom'] ) != 'active' )  {
        return false;
    }

    $user_screte = get_user_meta( get_current_user_id(), 'frontrom_secret_key', true );
    $user_screte = md5( $user_screte );
    
    if ( sanitize_text_field( $_GET['secret'] ) != $user_screte ) {
        return false;
    }

    if ( ! wp_verify_nonce( sanitize_text_field( $_GET['_wpnonce'] ) ) ) {
        return false;
    }

    return true;
}

 