<?php
namespace FrontRom\Includes;

class Scripts {

	private static $_instance;

    public static function instance() {
        if ( !self::$_instance ) {
            self::$_instance = new self();
        }

        return self::$_instance;
    }

	public function __construct() {
		add_filter( 'wp_footer', [ $this, 'scripts' ], 999999 );
		add_action( 'admin_enqueue_scripts', [$this, 'frontrom_scripts'] );
		add_action( 'enqueue_block_editor_assets', [ $this, 'block_editor_assets' ] );
	}

	public function block_editor_assets() { 
		
		wp_enqueue_style( 
			'frontrom-block-editor-assets', 
			FRONTROM_ASSET .'/css/block-editor-assets.css', 
			false, 
			filemtime( FRONTROM_ASSET_PATH . '/css/block-editor-assets.css' ), 
			'all' 
		);
		
		wp_enqueue_script( 'frontrom-block-editor-assets', FRONTROM_ASSET . '/js/block-editor-assets.js', [ 'jquery', 'wp-edit-post' ], filemtime( FRONTROM_ASSET_PATH . '/js/block-editor-assets.js' ), true );

		wp_localize_script( 'frontrom-block-editor-assets', 'FontRomBE',
			[
				'editLink' => $this->get_edit_url(),
			]
		);
	}
	

	public function frontrom_scripts() {
		wp_enqueue_script( 'frontrom', FRONTROM_ASSET . '/js/frontrom.js', [ 'jquery', 'wp-edit-post' ], filemtime( FRONTROM_ASSET_PATH . '/js/frontrom.js' ), true );

		wp_localize_script( 'frontrom', 'FrontRom',
			[
				//'editLink' => $this->get_edit_url(),
			]
		);
	}

	private function get_edit_url() {
		global $post;

		if ( empty( $post ) ) {
			return '';
		}
		
		$url = add_query_arg(
			[
				'post' => $post->ID,
				'action' => 'frontrom',
			],
			admin_url( 'post.php' )
		);

		return $url;
	}

	public function scripts() {
		if ( ! frontrom_user_can_access_front_end_editor() ) {
			return;
		}

		wp_enqueue_style( 
			'frontrom-frontend', 
			FRONTROM_ASSET .'/css/frontrom-frontend.css', 
			false, 
			filemtime( FRONTROM_ASSET_PATH . '/css/frontrom-frontend.css' ), 
			'all' 
		);

		$this->load_admin_scripts();
	}

	public function load_admin_scripts() {
		if ( ! isset( $_GET['frontrom_post_id'] ) ) {
			return false;
		}
		
		$post_id = empty( absint( $_GET['frontrom_post_id'] ) ) ? false : absint( $_GET['frontrom_post_id'] );
        $post    = get_post( $post_id );

        if ( empty( $post ) ) {
            return;
        }
        
		$this->register_all_admin_style();
		$this->register_all_admin_scripts();


		$post = get_post( $post->ID );

		wp_enqueue_script( 'heartbeat' );
		wp_enqueue_script( 'wp-edit-post' ); 
		wp_enqueue_script( 'wp-format-library' );

		$editor_settings = array(

			'disableCustomColors'    => get_theme_support( 'disable-custom-colors' ),
			'disableCustomFontSizes' => get_theme_support( 'disable-custom-font-sizes' ),
			'disablePostFormats'     => ! current_theme_supports( 'post-formats' ),
			/** This filter is documented in wp-admin/edit-form-advanced.php */
			'titlePlaceholder'       => apply_filters( 'enter_title_here', __( 'Add title' ), $post ),
			'isRTL'                  => is_rtl(),
			'autosaveInterval'       => AUTOSAVE_INTERVAL,
			'allowedMimeTypes'       => get_allowed_mime_types(),
			'richEditingEnabled'     => user_can_richedit(),
			'postLockUtils'          => array(
				'nonce'       => wp_create_nonce( 'lock-post_' . $post->ID ),
				'unlockNonce' => wp_create_nonce( 'update-post_' . $post->ID ),
				'ajaxUrl'     => admin_url( 'admin-ajax.php' ),
			),
			'enableCustomFields'     => (bool) get_user_meta( get_current_user_id(), 'enable_custom_fields', true ),
		);
			
		$init_script = "( function() {
			window._wpLoadBlockEditor = new Promise( function( resolve ) {
				wp.domReady( function() {
					resolve( wp.editPost.initializeEditor( 'editor', '%s', %d, %s, %s ) );
				} );
			} );
		} )();";
		$initial_edits = null;
		/**
		 * Scripts
		 */
		wp_enqueue_media(
			array(
				'post' => $post->ID,
			)
		);
		wp_tinymce_inline_scripts();


		wp_enqueue_editor();

		/**
		 * Styles
		 */
		wp_enqueue_style( 'wp-edit-post' );
		wp_enqueue_style( 'wp-format-library' );

		do_action( 'enqueue_block_assets' );
		do_action( 'enqueue_block_editor_assets' );
		
		wp_enqueue_script('coblocks-editor');
		wp_enqueue_style('coblocks-editor');


		$script = sprintf(
			$init_script,
			$post->post_type,
			$post->ID,
			wp_json_encode( $editor_settings ),
			wp_json_encode( $initial_edits )
		);
		wp_add_inline_script( 'wp-edit-post', $script );

		//wp_footer();
			/** This action is documented in wp-admin/admin-footer.php */
		print_late_styles();
		print_admin_styles();

		print_footer_scripts();
	}


	public function register_all_admin_scripts() {
		$scripts = new \WP_Scripts();
		
		$suffix     = wp_scripts_get_suffix();
		$dev_suffix = wp_scripts_get_suffix( 'dev' );
		$guessurl   = site_url();

		if ( ! $guessurl ) {
			$guessed_url = true;
			$guessurl    = wp_guess_url();
		}

		$scripts->base_url        = $guessurl;
		$scripts->content_url     = defined( 'WP_CONTENT_URL' ) ? WP_CONTENT_URL : '';
		$scripts->default_version = get_bloginfo( 'version' );
		$scripts->default_dirs    = array( $guessurl . '/wp-admin/js/', $guessurl . '/wp-includes/js/' );

		$this->add( 'utils', "$guessurl/wp-includes/js/utils$suffix.js" );

		$this->add( 'common', "$guessurl/wp-admin/js/common$suffix.js", array( 'jquery', 'hoverIntent', 'utils' ), false, 1 );
	

		$this->add( 'wp-sanitize', "$guessurl/wp-includes/js/wp-sanitize$suffix.js", array(), false, 1 );

		$this->add( 'wp-a11y', "$guessurl/wp-includes/js/wp-a11y$suffix.js", array( 'jquery', 'wp-sanitize' ), false, 1 );

		$this->add( 'sack', "$guessurl/wp-includes/js/tw-sack$suffix.js", array(), '1.6.1', 1 );

		$this->add( 'quicktags', "$guessurl/wp-includes/js/quicktags$suffix.js", array(), false, 1 );
		
		$this->add( 'colorpicker', "$guessurl/wp-includes/js/colorpicker$suffix.js", array( 'prototype' ), '3517m' );

		$this->add( 'editor', "$guessurl/wp-admin/js/editor$suffix.js", array( 'utils', 'jquery' ), false, 1 );

		$this->add( 'clipboard', "$guessurl/wp-includes/js/clipboard$suffix.js", array(), false, 1 );

		// Back-compat for old DFW. To-do: remove at the end of 2016.
		$this->add( 'wp-fullscreen-stub', "$guessurl/wp-admin/js/wp-fullscreen-stub$suffix.js", array(), false, 1 );

		$this->add( 'wp-ajax-response', "$guessurl/wp-includes/js/wp-ajax-response$suffix.js", array( 'jquery' ), false, 1 );
		

		$this->add( 'wp-api-request', "$guessurl/wp-includes/js/api-request$suffix.js", array( 'jquery' ), false, 1 );
		// `wpApiSettings` is also used by `wp-api`, which depends on this script.
		

		$this->add( 'wp-pointer', "$guessurl/wp-includes/js/wp-pointer$suffix.js", array( 'jquery-ui-widget', 'jquery-ui-position' ), '20111129a', 1 );
		

		$this->add( 'autosave', "$guessurl/wp-includes/js/autosave$suffix.js", array( 'heartbeat' ), false, 1 );

		$this->add( 'heartbeat', "$guessurl/wp-includes/js/heartbeat$suffix.js", array( 'jquery', 'wp-hooks' ), false, 1 );
		

		$this->add( 'wp-auth-check', "$guessurl/wp-includes/js/wp-auth-check$suffix.js", array( 'heartbeat' ), false, 1 );
		

		$this->add( 'wp-lists', "$guessurl/wp-includes/js/wp-lists$suffix.js", array( 'wp-ajax-response', 'jquery-color' ), false, 1 );

		// not used in core, replaced by Jcrop.js
		$this->add( 'cropper', "$guessurl/wp-includes/js/crop/cropper.js", array( 'scriptaculous-dragdrop' ) );

		// jQuery
		$this->add( 'jquery', false, array( 'jquery-core', 'jquery-migrate' ), '1.12.4-wp' );
		$this->add( 'jquery-core', $guessurl . '/wp-includes/js/jquery/jquery.js', array(), '1.12.4-wp' );
		$this->add( 'jquery-migrate', "$guessurl/wp-includes/js/jquery/jquery-migrate$suffix.js", array(), '1.4.1' );

		// full jQuery UI
		$this->add( 'jquery-ui-core', "$guessurl/wp-includes/js/jquery/ui/core$dev_suffix.js", array( 'jquery' ), '1.11.4', 1 );
		$this->add( 'jquery-effects-core', "$guessurl/wp-includes/js/jquery/ui/effect$dev_suffix.js", array( 'jquery' ), '1.11.4', 1 );

		$this->add( 'jquery-effects-blind', "$guessurl/wp-includes/js/jquery/ui/effect-blind$dev_suffix.js", array( 'jquery-effects-core' ), '1.11.4', 1 );
		$this->add( 'jquery-effects-bounce', "$guessurl/wp-includes/js/jquery/ui/effect-bounce$dev_suffix.js", array( 'jquery-effects-core' ), '1.11.4', 1 );
		$this->add( 'jquery-effects-clip', "$guessurl/wp-includes/js/jquery/ui/effect-clip$dev_suffix.js", array( 'jquery-effects-core' ), '1.11.4', 1 );
		$this->add( 'jquery-effects-drop', "$guessurl/wp-includes/js/jquery/ui/effect-drop$dev_suffix.js", array( 'jquery-effects-core' ), '1.11.4', 1 );
		$this->add( 'jquery-effects-explode', "$guessurl/wp-includes/js/jquery/ui/effect-explode$dev_suffix.js", array( 'jquery-effects-core' ), '1.11.4', 1 );
		$this->add( 'jquery-effects-fade', "$guessurl/wp-includes/js/jquery/ui/effect-fade$dev_suffix.js", array( 'jquery-effects-core' ), '1.11.4', 1 );
		$this->add( 'jquery-effects-fold', "$guessurl/wp-includes/js/jquery/ui/effect-fold$dev_suffix.js", array( 'jquery-effects-core' ), '1.11.4', 1 );
		$this->add( 'jquery-effects-highlight', "$guessurl/wp-includes/js/jquery/ui/effect-highlight$dev_suffix.js", array( 'jquery-effects-core' ), '1.11.4', 1 );
		$this->add( 'jquery-effects-puff', "$guessurl/wp-includes/js/jquery/ui/effect-puff$dev_suffix.js", array( 'jquery-effects-core', 'jquery-effects-scale' ), '1.11.4', 1 );
		$this->add( 'jquery-effects-pulsate', "$guessurl/wp-includes/js/jquery/ui/effect-pulsate$dev_suffix.js", array( 'jquery-effects-core' ), '1.11.4', 1 );
		$this->add( 'jquery-effects-scale', "$guessurl/wp-includes/js/jquery/ui/effect-scale$dev_suffix.js", array( 'jquery-effects-core', 'jquery-effects-size' ), '1.11.4', 1 );
		$this->add( 'jquery-effects-shake', "$guessurl/wp-includes/js/jquery/ui/effect-shake$dev_suffix.js", array( 'jquery-effects-core' ), '1.11.4', 1 );
		$this->add( 'jquery-effects-size', "$guessurl/wp-includes/js/jquery/ui/effect-size$dev_suffix.js", array( 'jquery-effects-core' ), '1.11.4', 1 );
		$this->add( 'jquery-effects-slide', "$guessurl/wp-includes/js/jquery/ui/effect-slide$dev_suffix.js", array( 'jquery-effects-core' ), '1.11.4', 1 );
		$this->add( 'jquery-effects-transfer', "$guessurl/wp-includes/js/jquery/ui/effect-transfer$dev_suffix.js", array( 'jquery-effects-core' ), '1.11.4', 1 );

		$this->add( 'jquery-ui-accordion', "$guessurl/wp-includes/js/jquery/ui/accordion$dev_suffix.js", array( 'jquery-ui-core', 'jquery-ui-widget' ), '1.11.4', 1 );
		$this->add( 'jquery-ui-autocomplete', "$guessurl/wp-includes/js/jquery/ui/autocomplete$dev_suffix.js", array( 'jquery-ui-menu', 'wp-a11y' ), '1.11.4', 1 );
		$this->add( 'jquery-ui-button', "$guessurl/wp-includes/js/jquery/ui/button$dev_suffix.js", array( 'jquery-ui-core', 'jquery-ui-widget' ), '1.11.4', 1 );
		$this->add( 'jquery-ui-datepicker', "$guessurl/wp-includes/js/jquery/ui/datepicker$dev_suffix.js", array( 'jquery-ui-core' ), '1.11.4', 1 );
		$this->add( 'jquery-ui-dialog', "$guessurl/wp-includes/js/jquery/ui/dialog$dev_suffix.js", array( 'jquery-ui-resizable', 'jquery-ui-draggable', 'jquery-ui-button', 'jquery-ui-position' ), '1.11.4', 1 );
		$this->add( 'jquery-ui-draggable', "$guessurl/wp-includes/js/jquery/ui/draggable$dev_suffix.js", array( 'jquery-ui-mouse' ), '1.11.4', 1 );
		$this->add( 'jquery-ui-droppable', "$guessurl/wp-includes/js/jquery/ui/droppable$dev_suffix.js", array( 'jquery-ui-draggable' ), '1.11.4', 1 );
		$this->add( 'jquery-ui-menu', "$guessurl/wp-includes/js/jquery/ui/menu$dev_suffix.js", array( 'jquery-ui-core', 'jquery-ui-widget', 'jquery-ui-position' ), '1.11.4', 1 );
		$this->add( 'jquery-ui-mouse', "$guessurl/wp-includes/js/jquery/ui/mouse$dev_suffix.js", array( 'jquery-ui-core', 'jquery-ui-widget' ), '1.11.4', 1 );
		$this->add( 'jquery-ui-position', "$guessurl/wp-includes/js/jquery/ui/position$dev_suffix.js", array( 'jquery' ), '1.11.4', 1 );
		$this->add( 'jquery-ui-progressbar', "$guessurl/wp-includes/js/jquery/ui/progressbar$dev_suffix.js", array( 'jquery-ui-core', 'jquery-ui-widget' ), '1.11.4', 1 );
		$this->add( 'jquery-ui-resizable', "$guessurl/wp-includes/js/jquery/ui/resizable$dev_suffix.js", array( 'jquery-ui-mouse' ), '1.11.4', 1 );
		$this->add( 'jquery-ui-selectable', "$guessurl/wp-includes/js/jquery/ui/selectable$dev_suffix.js", array( 'jquery-ui-mouse' ), '1.11.4', 1 );
		$this->add( 'jquery-ui-selectmenu', "$guessurl/wp-includes/js/jquery/ui/selectmenu$dev_suffix.js", array( 'jquery-ui-menu' ), '1.11.4', 1 );
		$this->add( 'jquery-ui-slider', "$guessurl/wp-includes/js/jquery/ui/slider$dev_suffix.js", array( 'jquery-ui-mouse' ), '1.11.4', 1 );
		$this->add( 'jquery-ui-sortable', "$guessurl/wp-includes/js/jquery/ui/sortable$dev_suffix.js", array( 'jquery-ui-mouse' ), '1.11.4', 1 );
		$this->add( 'jquery-ui-spinner', "$guessurl/wp-includes/js/jquery/ui/spinner$dev_suffix.js", array( 'jquery-ui-button' ), '1.11.4', 1 );
		$this->add( 'jquery-ui-tabs', "$guessurl/wp-includes/js/jquery/ui/tabs$dev_suffix.js", array( 'jquery-ui-core', 'jquery-ui-widget' ), '1.11.4', 1 );
		$this->add( 'jquery-ui-tooltip', "$guessurl/wp-includes/js/jquery/ui/tooltip$dev_suffix.js", array( 'jquery-ui-core', 'jquery-ui-widget', 'jquery-ui-position' ), '1.11.4', 1 );
		$this->add( 'jquery-ui-widget', "$guessurl/wp-includes/js/jquery/ui/widget$dev_suffix.js", array( 'jquery' ), '1.11.4', 1 );

		// deprecated, not used in core, most functionality is included in jQuery 1.3
		$this->add( 'jquery-form', "$guessurl/wp-includes/js/jquery/jquery.form$suffix.js", array( 'jquery' ), '4.2.1', 1 );

		// jQuery plugins
		$this->add( 'jquery-color', $guessurl . '/wp-includes/js/jquery/jquery.color.min.js', array( 'jquery' ), '2.1.1', 1 );
		$this->add( 'schedule', $guessurl . '/wp-includes/js/jquery/jquery.schedule.js', array( 'jquery' ), '20m', 1 );
		$this->add( 'jquery-query', $guessurl . '/wp-includes/js/jquery/jquery.query.js', array( 'jquery' ), '2.1.7', 1 );
		$this->add( 'jquery-serialize-object', $guessurl . '/wp-includes/js/jquery/jquery.serialize-object.js', array( 'jquery' ), '0.2', 1 );
		$this->add( 'jquery-hotkeys', "$guessurl/wp-includes/js/jquery/jquery.hotkeys$suffix.js", array( 'jquery' ), '0.0.2m', 1 );
		$this->add( 'jquery-table-hotkeys', "$guessurl/wp-includes/js/jquery/jquery.table-hotkeys$suffix.js", array( 'jquery', 'jquery-hotkeys' ), false, 1 );
		$this->add( 'jquery-touch-punch', $guessurl . '/wp-includes/js/jquery/jquery.ui.touch-punch.js', array( 'jquery-ui-widget', 'jquery-ui-mouse' ), '0.2.2', 1 );

		// Not used any more, registered for backward compatibility.
		$this->add( 'suggest', "$guessurl/wp-includes/js/jquery/suggest$suffix.js", array( 'jquery' ), '1.1-20110113', 1 );

		// Masonry v2 depended on jQuery. v3 does not. The older jquery-masonry handle is a shiv.
		// It sets jQuery as a dependency, as the theme may have been implicitly loading it this way.
		$this->add( 'imagesloaded', $guessurl . '/wp-includes/js/imagesloaded.min.js', array(), '3.2.0', 1 );
		$this->add( 'masonry', $guessurl . '/wp-includes/js/masonry.min.js', array( 'imagesloaded' ), '3.3.2', 1 );
		$this->add( 'jquery-masonry', "$guessurl/wp-includes/js/jquery/jquery.masonry$dev_suffix.js", array( 'jquery', 'masonry' ), '3.1.2b', 1 );

		$this->add( 'thickbox', $guessurl . '/wp-includes/js/thickbox/thickbox.js', array( 'jquery' ), '3.1-20121105', 1 );


		$this->add( 'jcrop', $guessurl . '/wp-includes/js/jcrop/jquery.Jcrop.min.js', array( 'jquery' ), '0.9.12' );

		$this->add( 'swfobject', $guessurl . '/wp-includes/js/swfobject.js', array(), '2.2-20120417' );

		// Error messages for Plupload.
		$uploader_l10n = array(
			'queue_limit_exceeded'      => __( 'You have attempted to queue too many files.' ),
			'file_exceeds_size_limit'   => __( '%s exceeds the maximum upload size for this site.' ),
			'zero_byte_file'            => __( 'This file is empty. Please try another.' ),
			'invalid_filetype'          => __( 'Sorry, this file type is not permitted for security reasons.' ),
			'not_an_image'              => __( 'This file is not an image. Please try another.' ),
			'image_memory_exceeded'     => __( 'Memory exceeded. Please try another smaller file.' ),
			'image_dimensions_exceeded' => __( 'This is larger than the maximum size. Please try another.' ),
			'default_error'             => __( 'An error occurred in the upload. Please try again later.' ),
			'missing_upload_url'        => __( 'There was a configuration error. Please contact the server administrator.' ),
			'upload_limit_exceeded'     => __( 'You may only upload 1 file.' ),
			'http_error'                => __( 'HTTP error.' ),
			'upload_failed'             => __( 'Upload failed.' ),
			/* translators: 1: Opening link tag, 2: Closing link tag */
			'big_upload_failed'         => __( 'Please try uploading this file with the %1$sbrowser uploader%2$s.' ),
			'big_upload_queued'         => __( '%s exceeds the maximum upload size for the multi-file uploader when used in your browser.' ),
			'io_error'                  => __( 'IO error.' ),
			'security_error'            => __( 'Security error.' ),
			'file_cancelled'            => __( 'File canceled.' ),
			'upload_stopped'            => __( 'Upload stopped.' ),
			'dismiss'                   => __( 'Dismiss' ),
			'crunching'                 => __( 'Crunching&hellip;' ),
			'deleted'                   => __( 'moved to the trash.' ),
			'error_uploading'           => __( '&#8220;%s&#8221; has failed to upload.' ),
		);

		$this->add( 'moxiejs', "$guessurl/wp-includes/js/plupload/moxie$suffix.js", array(), '1.3.5' );
		$this->add( 'plupload', "$guessurl/wp-includes/js/plupload/plupload$suffix.js", array( 'moxiejs' ), '2.1.9' );
		// Back compat handles:
		foreach ( array( 'all', 'html5', 'flash', 'silverlight', 'html4' ) as $handle ) {
			$this->add( "plupload-$handle", false, array( 'plupload' ), '2.1.1' );
		}

		$this->add( 'plupload-handlers', "$guessurl/wp-includes/js/plupload/handlers$suffix.js", array( 'plupload', 'jquery' ) );
		

		$this->add( 'wp-plupload', "$guessurl/wp-includes/js/plupload/wp-plupload$suffix.js", array( 'plupload', 'jquery', 'json2', 'media-models' ), false, 1 );
		

		// keep 'swfupload' for back-compat.
		$this->add( 'swfupload', $guessurl . '/wp-includes/js/swfupload/swfupload.js', array(), '2201-20110113' );
		$this->add( 'swfupload-all', false, array( 'swfupload' ), '2201' );
		$this->add( 'swfupload-handlers', "$guessurl/wp-includes/js/swfupload/handlers$suffix.js", array( 'swfupload-all', 'jquery' ), '2201-20110524' );
		

		$this->add( 'comment-reply', "$guessurl/wp-includes/js/comment-reply$suffix.js", array(), false, 1 );

		$this->add( 'json2', "$guessurl/wp-includes/js/json2$suffix.js", array(), '2015-05-03' );
		

		$this->add( 'underscore', "$guessurl/wp-includes/js/underscore$dev_suffix.js", array(), '1.8.3', 1 );
		$this->add( 'backbone', "$guessurl/wp-includes/js/backbone$dev_suffix.js", array( 'underscore', 'jquery' ), '1.2.3', 1 );

		$this->add( 'wp-util', "$guessurl/wp-includes/js/wp-util$suffix.js", array( 'underscore', 'jquery' ), false, 1 );


		$this->add( 'wp-backbone', "$guessurl/wp-includes/js/wp-backbone$suffix.js", array( 'backbone', 'wp-util' ), false, 1 );

		$this->add( 'revisions', "$guessurl/wp-admin/js/revisions$suffix.js", array( 'wp-backbone', 'jquery-ui-slider', 'hoverIntent' ), false, 1 );

		$this->add( 'imgareaselect', "$guessurl/wp-includes/js/imgareaselect/jquery.imgareaselect$suffix.js", array( 'jquery' ), false, 1 );

		$this->add( 'mediaelement', false, array( 'jquery', 'mediaelement-core', 'mediaelement-migrate' ), '4.2.6-78496d1' );
		$this->add( 'mediaelement-core', "$guessurl/wp-includes/js/mediaelement/mediaelement-and-player$suffix.js", array(), '4.2.6-78496d1', 1 );
		$this->add( 'mediaelement-migrate', "$guessurl/wp-includes/js/mediaelement/mediaelement-migrate$suffix.js", array(), false, 1 );

		

		$this->add( 'mediaelement-vimeo', $guessurl . '/wp-includes/js/mediaelement/renderers/vimeo.min.js', array( 'mediaelement' ), '4.2.6-78496d1', 1 );
		$this->add( 'wp-mediaelement', "$guessurl/wp-includes/js/mediaelement/wp-mediaelement$suffix.js", array( 'mediaelement' ), false, 1 );
		

		$this->add( 'wp-codemirror', $guessurl . '/wp-includes/js/codemirror/codemirror.min.js', array(), '5.29.1-alpha-ee20357' );
		$this->add( 'csslint', $guessurl . '/wp-includes/js/codemirror/csslint.js', array(), '1.0.5' );
		$this->add( 'esprima', $guessurl . '/wp-includes/js/codemirror/esprima.js', array(), '4.0.0' );
		$this->add( 'jshint', $guessurl . '/wp-includes/js/codemirror/fakejshint.js', array( 'esprima' ), '2.9.5' );
		$this->add( 'jsonlint', $guessurl . '/wp-includes/js/codemirror/jsonlint.js', array(), '1.6.2' );
		$this->add( 'htmlhint', $guessurl . '/wp-includes/js/codemirror/htmlhint.js', array(), '0.9.14-xwp' );
		$this->add( 'htmlhint-kses', $guessurl . '/wp-includes/js/codemirror/htmlhint-kses.js', array( 'htmlhint' ) );
		$this->add( 'code-editor', "$guessurl/wp-admin/js/code-editor$suffix.js", array( 'jquery', 'wp-codemirror', 'underscore' ) );
		$this->add( 'wp-theme-plugin-editor', "$guessurl/wp-admin/js/theme-plugin-editor$suffix.js", array( 'wp-util', 'wp-sanitize', 'jquery', 'jquery-ui-core', 'wp-a11y', 'underscore' ) );
		

		$this->add( 'wp-playlist', "$guessurl/wp-includes/js/mediaelement/wp-playlist$suffix.js", array( 'wp-util', 'backbone', 'mediaelement' ), false, 1 );

		$this->add( 'zxcvbn-async', "$guessurl/wp-includes/js/zxcvbn-async$suffix.js", array(), '1.0' );
		

		$this->add( 'password-strength-meter', "$guessurl/wp-admin/js/password-strength-meter$suffix.js", array( 'jquery', 'zxcvbn-async' ), false, 1 );
		

		$this->add( 'user-profile', "$guessurl/wp-admin/js/user-profile$suffix.js", array( 'jquery', 'password-strength-meter', 'wp-util' ), false, 1 );
		

		$this->add( 'language-chooser', "$guessurl/wp-admin/js/language-chooser$suffix.js", array( 'jquery' ), false, 1 );

		$this->add( 'user-suggest', "$guessurl/wp-admin/js/user-suggest$suffix.js", array( 'jquery-ui-autocomplete' ), false, 1 );

		$this->add( 'admin-bar', "$guessurl/wp-includes/js/admin-bar$suffix.js", array(), false, 1 );

		$this->add( 'wplink', "$guessurl/wp-includes/js/wplink$suffix.js", array( 'jquery', 'wp-a11y' ), false, 1 );
		

		$this->add( 'wpdialogs', "$guessurl/wp-includes/js/wpdialog$suffix.js", array( 'jquery-ui-dialog' ), false, 1 );

		$this->add( 'word-count', "$guessurl/wp-admin/js/word-count$suffix.js", array(), false, 1 );

		$this->add( 'media-upload', "$guessurl/wp-admin/js/media-upload$suffix.js", array( 'thickbox', 'shortcode' ), false, 1 );

		$this->add( 'hoverIntent', "$guessurl/wp-includes/js/hoverIntent$suffix.js", array( 'jquery' ), '1.8.1', 1 );

		$this->add( 'customize-base', "$guessurl/wp-includes/js/customize-base$suffix.js", array( 'jquery', 'json2', 'underscore' ), false, 1 );
		$this->add( 'customize-loader', "$guessurl/wp-includes/js/customize-loader$suffix.js", array( 'customize-base' ), false, 1 );
		$this->add( 'customize-preview', "$guessurl/wp-includes/js/customize-preview$suffix.js", array( 'wp-a11y', 'customize-base' ), false, 1 );
		$this->add( 'customize-models', $guessurl . '/wp-includes/js/customize-models.js', array( 'underscore', 'backbone' ), false, 1 );
		$this->add( 'customize-views', $guessurl . '/wp-includes/js/customize-views.js', array( 'jquery', 'underscore', 'imgareaselect', 'customize-models', 'media-editor', 'media-views' ), false, 1 );
		$this->add( 'customize-controls', "$guessurl/wp-admin/js/customize-controls$suffix.js", array( 'customize-base', 'wp-a11y', 'wp-util', 'jquery-ui-core' ), false, 1 );
		
		$this->add( 'customize-selective-refresh', "$guessurl/wp-includes/js/customize-selective-refresh$suffix.js", array( 'jquery', 'wp-util', 'customize-preview' ), false, 1 );

		$this->add( 'customize-widgets', "$guessurl/wp-admin/js/customize-widgets$suffix.js", array( 'jquery', 'jquery-ui-sortable', 'jquery-ui-droppable', 'wp-backbone', 'customize-controls' ), false, 1 );
		$this->add( 'customize-preview-widgets', "$guessurl/wp-includes/js/customize-preview-widgets$suffix.js", array( 'jquery', 'wp-util', 'customize-preview', 'customize-selective-refresh' ), false, 1 );

		$this->add( 'customize-nav-menus', "$guessurl/wp-admin/js/customize-nav-menus$suffix.js", array( 'jquery', 'wp-backbone', 'customize-controls', 'accordion', 'nav-menu', 'wp-sanitize' ), false, 1 );
		$this->add( 'customize-preview-nav-menus', "$guessurl/wp-includes/js/customize-preview-nav-menus$suffix.js", array( 'jquery', 'wp-util', 'customize-preview', 'customize-selective-refresh' ), false, 1 );

		$this->add( 'wp-custom-header', "$guessurl/wp-includes/js/wp-custom-header$suffix.js", array( 'wp-a11y' ), false, 1 );

		$this->add( 'accordion', "$guessurl/wp-admin/js/accordion$suffix.js", array( 'jquery' ), false, 1 );

		$this->add( 'shortcode', "$guessurl/wp-includes/js/shortcode$suffix.js", array( 'underscore' ), false, 1 );
		$this->add( 'media-models', "$guessurl/wp-includes/js/media-models$suffix.js", array( 'wp-backbone' ), false, 1 );
		

		$this->add( 'wp-embed', "$guessurl/wp-includes/js/wp-embed$suffix.js" );

		// To enqueue media-views or media-editor, call wp_enqueue_media().
		// Both rely on numerous settings, styles, and templates to operate correctly.
		$this->add( 'media-views', "$guessurl/wp-includes/js/media-views$suffix.js", array( 'utils', 'media-models', 'wp-plupload', 'jquery-ui-sortable', 'wp-mediaelement', 'wp-api-request' ), false, 1 );
		$this->add( 'media-editor', "$guessurl/wp-includes/js/media-editor$suffix.js", array( 'shortcode', 'media-views' ), false, 1 );
		$this->add( 'media-audiovideo', "$guessurl/wp-includes/js/media-audiovideo$suffix.js", array( 'media-editor' ), false, 1 );
		$this->add( 'mce-view', "$guessurl/wp-includes/js/mce-view$suffix.js", array( 'shortcode', 'jquery', 'media-views', 'media-audiovideo' ), false, 1 );

		$this->add( 'wp-api', "$guessurl/wp-includes/js/wp-api$suffix.js", array( 'jquery', 'backbone', 'underscore', 'wp-api-request' ), false, 1 );

		
		$this->add( 'admin-tags', "$guessurl/wp-admin/js/tags$suffix.js", array( 'jquery', 'wp-ajax-response' ), false, 1 );
		

		$this->add( 'admin-comments', "$guessurl/wp-admin/js/edit-comments$suffix.js", array( 'wp-lists', 'quicktags', 'jquery-query' ), false, 1 );
		

		$this->add( 'xfn', "$guessurl/wp-admin/js/xfn$suffix.js", array( 'jquery' ), false, 1 );
		

		$this->add( 'postbox', "$guessurl/wp-admin/js/postbox$suffix.js", array( 'jquery-ui-sortable' ), false, 1 ); 

	

		$this->add( 'tags-box', "$guessurl/wp-admin/js/tags-box$suffix.js", array( 'jquery', 'tags-suggest' ), false, 1 );

		$this->add( 'tags-suggest', "$guessurl/wp-admin/js/tags-suggest$suffix.js", array( 'jquery-ui-autocomplete', 'wp-a11y' ), false, 1 );
		

		$this->add( 'post', "$guessurl/wp-admin/js/post$suffix.js", array( 'suggest', 'wp-lists', 'postbox', 'tags-box', 'underscore', 'word-count', 'wp-a11y', 'wp-sanitize' ), false, 1 );
		

		$this->add( 'editor-expand', "$guessurl/wp-admin/js/editor-expand$suffix.js", array( 'jquery', 'underscore' ), false, 1 );

		$this->add( 'link', "$guessurl/wp-admin/js/link$suffix.js", array( 'wp-lists', 'postbox' ), false, 1 );

		$this->add( 'comment', "$guessurl/wp-admin/js/comment$suffix.js", array( 'jquery', 'postbox' ) );
		//$this->add_data( 'comment', 'group', 1 );
		

		$this->add( 'admin-gallery', "$guessurl/wp-admin/js/gallery$suffix.js", array( 'jquery-ui-sortable' ) );

		$this->add( 'admin-widgets', "$guessurl/wp-admin/js/widgets$suffix.js", array( 'jquery-ui-sortable', 'jquery-ui-draggable', 'jquery-ui-droppable' ), false, 1 );
		

		$this->add( 'media-widgets', "$guessurl/wp-admin/js/widgets/media-widgets$suffix.js", array( 'jquery', 'media-models', 'media-views', 'wp-api-request' ) );
		//$this->add_inline_script( 'media-widgets', 'wp.mediaWidgets.init();', 'after' );

		$this->add( 'media-audio-widget', "$guessurl/wp-admin/js/widgets/media-audio-widget$suffix.js", array( 'media-widgets', 'media-audiovideo' ) );
		$this->add( 'media-image-widget', "$guessurl/wp-admin/js/widgets/media-image-widget$suffix.js", array( 'media-widgets' ) );
		$this->add( 'media-gallery-widget', "$guessurl/wp-admin/js/widgets/media-gallery-widget$suffix.js", array( 'media-widgets' ) );
		$this->add( 'media-video-widget', "$guessurl/wp-admin/js/widgets/media-video-widget$suffix.js", array( 'media-widgets', 'media-audiovideo', 'wp-api-request' ) );
		$this->add( 'text-widgets', "$guessurl/wp-admin/js/widgets/text-widgets$suffix.js", array( 'jquery', 'backbone', 'editor', 'wp-util', 'wp-a11y' ) );
		$this->add( 'custom-html-widgets', "$guessurl/wp-admin/js/widgets/custom-html-widgets$suffix.js", array( 'jquery', 'backbone', 'wp-util', 'jquery-ui-core', 'wp-a11y' ) );

		$this->add( 'theme', "$guessurl/wp-admin/js/theme$suffix.js", array( 'wp-backbone', 'wp-a11y', 'customize-base' ), false, 1 );

		$this->add( 'inline-edit-post', "$guessurl/wp-admin/js/inline-edit-post$suffix.js", array( 'jquery', 'tags-suggest', 'wp-a11y' ), false, 1 );
		

		$this->add( 'inline-edit-tax', "$guessurl/wp-admin/js/inline-edit-tax$suffix.js", array( 'jquery', 'wp-a11y' ), false, 1 );
		

		$this->add( 'plugin-install', "$guessurl/wp-admin/js/plugin-install$suffix.js", array( 'jquery', 'jquery-ui-core', 'thickbox' ), false, 1 );
		

		$this->add( 'site-health', "$guessurl/wp-admin/js/site-health$suffix.js", array( 'clipboard', 'jquery', 'wp-util', 'wp-a11y', 'wp-i18n' ), false, 1 );
		$scripts->set_translations( 'site-health' );

		$this->add( 'updates', "$guessurl/wp-admin/js/updates$suffix.js", array( 'jquery', 'wp-util', 'wp-a11y', 'wp-sanitize' ), false, 1 );
		

		$this->add( 'farbtastic', $guessurl . '/wp-admin/js/farbtastic.js', array( 'jquery' ), '1.2' );

		$this->add( 'iris', $guessurl . '/wp-admin/js/iris.min.js', array( 'jquery-ui-draggable', 'jquery-ui-slider', 'jquery-touch-punch' ), '1.0.7', 1 );
		$this->add( 'wp-color-picker', "$guessurl/wp-admin/js/color-picker$suffix.js", array( 'iris' ), false, 1 );


		$this->add( 'dashboard', "$guessurl/wp-admin/js/dashboard$suffix.js", array( 'jquery', 'admin-comments', 'postbox', 'wp-util', 'wp-a11y' ), false, 1 );

		$this->add( 'list-revisions', "$guessurl/wp-includes/js/wp-list-revisions$suffix.js" );

		$this->add( 'media-grid', "$guessurl/wp-includes/js/media-grid$suffix.js", array( 'media-editor' ), false, 1 );
		$this->add( 'media', "$guessurl/wp-admin/js/media$suffix.js", array( 'jquery' ), false, 1 );
		

		$this->add( 'image-edit', "$guessurl/wp-admin/js/image-edit$suffix.js", array( 'jquery', 'json2', 'imgareaselect' ), false, 1 );
	

		$this->add( 'set-post-thumbnail', "$guessurl/wp-admin/js/set-post-thumbnail$suffix.js", array( 'jquery' ), false, 1 );
	

		/*
		 * Navigation Menus: Adding underscore as a dependency to utilize _.debounce
		 * see https://core.trac.wordpress.org/ticket/42321
		 */
		$this->add( 'nav-menu', "$guessurl/wp-admin/js/nav-menu$suffix.js", array( 'jquery-ui-sortable', 'jquery-ui-draggable', 'jquery-ui-droppable', 'wp-lists', 'postbox', 'json2', 'underscore' ) );
		

		$this->add( 'custom-header', $guessurl . '/wp-admin/js/custom-header.js', array( 'jquery-masonry' ), false, 1 );
		$this->add( 'custom-background', "$guessurl/wp-admin/js/custom-background$suffix.js", array( 'wp-color-picker', 'media-views' ), false, 1 );
		$this->add( 'media-gallery', "$guessurl/wp-admin/js/media-gallery$suffix.js", array( 'jquery' ), false, 1 );

		$this->add( 'svg-painter', $guessurl . '/wp-admin/js/svg-painter.js', array( 'jquery' ), false, 1 );
		
	}



	public function add( $handel, $src, $dependency = [], $tagb = false, $is_footer = false ) {
		wp_register_script( $handel, $src, $dependency, $tagb, $is_footer );
	}

	public function register_all_admin_style() {
		$styles = new \WP_Styles();
		include( ABSPATH . WPINC . '/version.php' ); // include an unmodified $wp_version

		if ( ! defined( 'SCRIPT_DEBUG' ) ) {
			define( 'SCRIPT_DEBUG', false !== strpos( $wp_version, '-src' ) );
		}

		$guessurl = site_url();

		if ( ! $guessurl ) {
			$guessurl = wp_guess_url();
		}

		$styles->base_url        = $guessurl;
		$styles->content_url     = defined( 'WP_CONTENT_URL' ) ? WP_CONTENT_URL : '';
		$styles->default_version = get_bloginfo( 'version' );
		$styles->text_direction  = function_exists( 'is_rtl' ) && is_rtl() ? 'rtl' : 'ltr';
		$styles->default_dirs    = array( $guessurl . '/wp-admin/', $guessurl . '/wp-includes/css/' );

		// Open Sans is no longer used by core, but may be relied upon by themes and plugins.
		$open_sans_font_url = '';

		// Register a stylesheet for the selected admin color scheme.
		//$this->add_style( 'colors', true, array( 'wp-admin', 'buttons' ) );
		$suffix = SCRIPT_DEBUG ? '' : '.min';
			// Admin CSS
		$this->add_style( 'common', "$guessurl/wp-admin/css/common$suffix.css" );
		$this->add_style( 'forms', "$guessurl/wp-admin/css/forms$suffix.css" );
		$this->add_style( 'admin-menu', "$guessurl/wp-admin/css/admin-menu$suffix.css" );
		$this->add_style( 'dashboard', "$guessurl/wp-admin/css/dashboard$suffix.css" );
		$this->add_style( 'list-tables', "$guessurl/wp-admin/css/list-tables$suffix.css" );
		$this->add_style( 'edit', "$guessurl/wp-admin/css/edit$suffix.css" );
		$this->add_style( 'revisions', "$guessurl/wp-admin/css/revisions$suffix.css" );
		$this->add_style( 'media', "$guessurl/wp-admin/css/media$suffix.css" );
		$this->add_style( 'themes', "$guessurl/wp-admin/css/themes$suffix.css" );
		$this->add_style( 'about', "$guessurl/wp-admin/css/about$suffix.css" );
		$this->add_style( 'nav-menus', "$guessurl/wp-admin/css/nav-menus$suffix.css" );
		$this->add_style( 'widgets', "$guessurl/wp-admin/css/widgets$suffix.css", array( 'wp-pointer' ) );
		$this->add_style( 'site-icon', "$guessurl/wp-admin/css/site-icon$suffix.css" );
		$this->add_style( 'l10n', "$guessurl/wp-admin/css/l10n$suffix.css" );
		$this->add_style( 'code-editor', "$guessurl/wp-admin/css/code-editor$suffix.css", array( 'wp-codemirror' ) );
		$this->add_style( 'site-health', "$guessurl/wp-admin/css/site-health$suffix.css" );

		//$this->add_style( 'wp-admin', false, array( 'dashicons', 'common', 'forms', 'admin-menu', 'dashboard', 'list-tables', 'edit', 'revisions', 'media', 'themes', 'about', 'nav-menus', 'widgets', 'site-icon', 'l10n' ) );

		$this->add_style( 'login', "$guessurl/wp-admin/css/login$suffix.css", array( 'dashicons', 'buttons', 'forms', 'l10n' ) );
		$this->add_style( 'install', "$guessurl/wp-admin/css/install$suffix.css", array( 'dashicons', 'buttons', 'forms', 'l10n' ) );
		$this->add_style( 'wp-color-picker', "$guessurl/wp-admin/css/color-picker$suffix.css" );
		$this->add_style( 'customize-controls', "$guessurl/wp-admin/css/customize-controls$suffix.css", array( 'wp-admin', 'colors', 'ie', 'imgareaselect' ) );
		$this->add_style( 'customize-widgets', "$guessurl/wp-admin/css/customize-widgets$suffix.css", array( 'wp-admin', 'colors' ) );
		$this->add_style( 'customize-nav-menus', "$guessurl/wp-admin/css/customize-nav-menus$suffix.css", array( 'wp-admin', 'colors' ) );

		$this->add_style( 'ie', "$guessurl/wp-admin/css/ie$suffix.css" );
		

		// Common dependencies
		$this->add_style( 'buttons', "$guessurl/wp-includes/css/buttons$suffix.css" );
		$this->add_style( 'dashicons', "$guessurl/wp-includes/css/dashicons$suffix.css" );

		// Includes CSS
		$this->add_style( 'admin-bar', "$guessurl/wp-includes/css/admin-bar$suffix.css", array( 'dashicons' ) );
		$this->add_style( 'wp-auth-check', "$guessurl/wp-includes/css/wp-auth-check$suffix.css", array( 'dashicons' ) );
		$this->add_style( 'editor-buttons', "$guessurl/wp-includes/css/editor$suffix.css", array( 'dashicons' ) );
		$this->add_style( 'media-views', "$guessurl/wp-includes/css/media-views$suffix.css", array( 'buttons', 'dashicons', 'wp-mediaelement' ) );
		$this->add_style( 'wp-pointer', "$guessurl/wp-includes/css/wp-pointer$suffix.css", array( 'dashicons' ) );
		$this->add_style( 'customize-preview', "$guessurl/wp-includes/css/customize-preview$suffix.css", array( 'dashicons' ) );
		$this->add_style( 'wp-embed-template-ie', "$guessurl/wp-includes/css/wp-embed-template-ie$suffix.css" );
		

		// External libraries and friends
		$this->add_style( 'imgareaselect', $guessurl . '/wp-includes/js/imgareaselect/imgareaselect.css', array(), '0.9.8' );
		$this->add_style( 'wp-jquery-ui-dialog', "$guessurl/wp-includes/css/jquery-ui-dialog$suffix.css", array( 'dashicons' ) );
		$this->add_style( 'mediaelement', $guessurl . '/wp-includes/js/mediaelement/mediaelementplayer-legacy.min.css', array(), '4.2.6-78496d1' );
		$this->add_style( 'wp-mediaelement', "$guessurl/wp-includes/js/mediaelement/wp-mediaelement$suffix.css", array( 'mediaelement' ) );
		$this->add_style( 'thickbox', $guessurl . '/wp-includes/js/thickbox/thickbox.css', array( 'dashicons' ) );
		$this->add_style( 'wp-codemirror', $guessurl . '/wp-includes/js/codemirror/codemirror.min.css', array(), '5.29.1-alpha-ee20357' );

		// Deprecated CSS
		$this->add_style( 'deprecated-media', "$guessurl/wp-admin/css/deprecated-media$suffix.css" );
		$this->add_style( 'farbtastic', "$guessurl/wp-admin/css/farbtastic$suffix.css", array(), '1.3u1' );
		$this->add_style( 'jcrop', $guessurl . '/wp-includes/js/jcrop/jquery.Jcrop.min.css', array(), '0.9.12' );
		$this->add_style( 'colors-fresh', false, array( 'wp-admin', 'buttons' ) ); // Old handle.
		
		$package_styles = array(
			'block-editor'         => array(
				'wp-components',
				'wp-editor-font',
			),
			'block-library'        => array(),
			'components'           => array(),
			'edit-post'            => array(
				'wp-components',
				'wp-block-editor',
				'wp-editor',
				'wp-edit-blocks',
				'wp-block-library',
				'wp-nux',
			),
			'editor'               => array(
				'wp-components',
				'wp-block-editor',
				'wp-nux',
			),
			'format-library'       => array(),
			'list-reusable-blocks' => array( 'wp-components' ),
			'nux'                  => array( 'wp-components' ),
		);

		foreach ( $package_styles as $package => $dependencies ) {
			$handle = 'wp-' . $package;
			$path   = "$guessurl/wp-includes/css/dist/$package/style$suffix.css";

			$this->add_style( $handle, $path, $dependencies );
		}


	}

	public function add_style( $handle, $src, $deps = array(), $ver = false, $media = 'all' ) {
		//$src = "http://localhost/gutenberg" . $src;
		wp_register_style( $handle, $src, $deps, $ver, $media );
	}
}